// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef AEVOL_ANNOTATED_CHROMOSOME_H
#define AEVOL_ANNOTATED_CHROMOSOME_H

#include <cinttypes>

#include <list>

#include "Dna_7.h"
#include "DnaFactory.h"
#include "PromoterList.h"
#include "ProteinList.h"
#include "RnaList.h"

namespace aevol {

class Individual_7;
class List_Metadata;

class AnnotatedChromosome {
 public:
  AnnotatedChromosome(const AnnotatedChromosome&) = delete;
  AnnotatedChromosome(AnnotatedChromosome&&) = delete;
  AnnotatedChromosome& operator=(const AnnotatedChromosome&) = delete;
  AnnotatedChromosome& operator=(AnnotatedChromosome&&) = delete;
  ~AnnotatedChromosome();

 private:
  AnnotatedChromosome();
  AnnotatedChromosome(const char* seq, Dna_7::size_type len);

 public:
  static AnnotatedChromosome* make_empty();
  static AnnotatedChromosome* make_from_sequence(const char* seq, Dna_7::size_type len);
  static AnnotatedChromosome* make_clone(AnnotatedChromosome* orig, Individual_7* owner);

  const std::list<Rna_7*>& rnas() const;

  PromoterList& promoter_list();
  PromoterList::SingleStranded& promoter_list(Strand strand);

  const std::list<Protein_7*>& proteins() const;

  void compute_non_coding();

  void reset_stats();

  void reset_metadata();

  Dna_7* dna_ = nullptr;
  List_Metadata* metadata_;
  PromoterList promoter_list_;
  RnaList rna_list_;
  ProteinList proteins_;

  bool non_coding_computed_ = false;

  /** Variables for Tree mgmt **/
  int32_t nb_genes_activ     = 0;
  int32_t nb_genes_inhib     = 0;
  int32_t nb_func_genes      = 0;
  int32_t nb_non_func_genes  = 0;
  int32_t nb_coding_RNAs     = 0;
  int32_t nb_non_coding_RNAs = 0;
  /** END of Variables for Tree Mgmt **/

  int32_t nb_bases_in_0_CDS_;
  int32_t nb_bases_in_0_functional_CDS_;
  int32_t nb_bases_in_0_non_functional_CDS_;
  int32_t nb_bases_in_0_RNA_;
  int32_t nb_bases_in_0_coding_RNA_;
  int32_t nb_bases_in_0_non_coding_RNA_;
  int32_t nb_bases_non_essential_;
  int32_t nb_bases_non_essential_including_nf_genes_;
  int32_t nb_bases_in_neutral_regions_;
};

} // namespace aevol

#endif // AEVOL_ANNOTATED_CHROMOSOME_H
