// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************


#ifndef AEVOL_LIST_METADATA_H
#define AEVOL_LIST_METADATA_H

#include "Protein_7.h"
#include "Rna_7.h"
#include "Dna_7.h"
#include "biochemistry/Strand.h"


#include <algorithm>

namespace aevol {

class List_Metadata {
    public:
     List_Metadata(AnnotatedChromosome* ann_chrsm);

     ~List_Metadata();

     /*** Promoters ***/
     static int8_t is_promoter_leading(const Dna_7& dna, int pos);
     static int8_t is_promoter_lagging(const Dna_7& dna, int pos);

     /*** Terminators ***/
     #ifdef BASE_4
       bool is_terminator_leading(int pos);
       bool is_terminator_lagging(int pos);
     #endif

     /*** Shine Dal + Start Codon ***/
     int8_t is_shine_dal_start_prot_leading(int pos);
     int8_t is_shine_dal_start_prot_lagging(int pos);

        /** Search and update **/

        int32_t length();

  AnnotatedChromosome* ann_chrsm_;
 protected:
};

}


#endif //AEVOL_LIST_METADATA_H
