// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef AEVOL_PROTEINLIST_H_
#define AEVOL_PROTEINLIST_H_

#include <list>

#include "Protein_7.h"

namespace aevol {

class ProteinList {
 public:
  ProteinList()                              = default;
  ProteinList(const ProteinList&)            = delete;
  ProteinList(ProteinList&&)                 = delete;
  ProteinList& operator=(const ProteinList&) = delete;
  ProteinList& operator=(ProteinList&&)      = delete;
  virtual ~ProteinList();

  inline const std::list<Protein_7*>& proteins() const;

  void add(Protein_7* prot);
  void clear();

  #ifdef __REGUL
    inline std::vector<Protein_7*> signal_proteins(); // TODO<dpa> non-const!
    inline void add_inherited_protein(Protein_7* protein);
    void add_inherited_proteins();
    void add_signal_proteins(std::vector<Protein_7*> signal_models);
    inline void add_translated_protein(Protein_7* protein);
    inline void set_signal_proteins_concentration(const std::list<int16_t>& signal_ids, double value);
    inline void reset_signal_proteins_concentration();
    void reinit_proteins_inherited();
  #endif

 protected:
  // Main protein list - contains all the proteins regardless of their origin
  std::list<Protein_7*> proteins_;

  #ifdef __REGUL
    // Inherited proteins - contains proteins that were inherited from the parent
    std::list<Protein_7*> inherited_proteins_;
    // Whether inherited proteins have been added to the main protein list
    bool inherited_added_ = false;
    // Signal proteins - environmental signal proteins
    std::vector<Protein_7*> signal_proteins_;
    // Translated proteins - those proteins that are encoded on the genome
    std::list<Protein_7*> translated_proteins_;
  #endif
};

const std::list<Protein_7*>& ProteinList::proteins() const {
  return proteins_;
}

#ifdef __REGUL
std::vector<Protein_7*> ProteinList::signal_proteins() {
  return signal_proteins_;
}

void ProteinList::add_inherited_protein(Protein_7* protein) {
  inherited_proteins_.push_back(protein);
}

void ProteinList::add_translated_protein(Protein_7* protein) {
  add(protein);
  translated_proteins_.push_front(protein);
}

void ProteinList::set_signal_proteins_concentration(const std::list<int16_t>& signal_ids, double value) {
  for (const auto& signal_id : signal_ids) {
    signal_proteins_[signal_id]->e = value;
  }
}

void ProteinList::reset_signal_proteins_concentration() {
  for (auto& protein : signal_proteins_) {
    protein->e = 0.;
  }
}
#endif

} // namespace aevol

#endif  // AEVOL_PROTEINLIST_H_
