// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef AEVOL_RNALIST_H_
#define AEVOL_RNALIST_H_

#include <cinttypes>

#include "Promoter.h"
#include "Rna_7.h"
#include "Strand.h"

namespace aevol {

class RnaList {
 public:
  RnaList()                          = default;
  RnaList(const RnaList&)            = delete;
  RnaList(RnaList&&)                 = delete;
  RnaList& operator=(const RnaList&) = delete;
  RnaList& operator=(RnaList&&)      = delete;
  virtual ~RnaList();

  const std::list<Rna_7*>& rnas() const;

  void rna_add(int32_t t_begin, int32_t t_end, Strand strand, double t_e, int32_t t_length, Promoter* prom = nullptr);
  void clear();

 protected:
  std::list<Rna_7*> rnas_;
};

} // namespace aevol

#endif  // AEVOL_RNALIST_H_
