// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#include "BindingMatrix.h"

#include <iostream>
#include <string>

namespace aevol {

std::unique_ptr<BindingMatrix> BindingMatrix::instance_ = nullptr;

static std::string binding_matrix_file_name = "binding_matrix.rae";

void BindingMatrix::random_binding_matrix(double binding_zeros_percentage, std::shared_ptr<JumpingMT> prng) {
  assert(instance_ == nullptr);

  instance_ = std::make_unique<BindingMatrix>();
  for (int8_t i = 0; i < MAX_QUADON; i++) {
    for (int8_t j = 0; j < MAX_CODON; j++) {
      if (prng->random() > binding_zeros_percentage) {
        instance_->binding_matrix_[i][j] = static_cast<binding_intensity_t>(prng->random());
      } else {
        instance_->binding_matrix_[i][j] = 0;
      }
    }
  }
}

void BindingMatrix::save() {
  // Open file
  gzFile binding_matrix_file = gzopen(binding_matrix_file_name.c_str(), "w");
  if (binding_matrix_file == Z_NULL) {
    std::cout << "ERROR : Could not write binding matrix file " << binding_matrix_file_name << std::endl;
    exit(EXIT_FAILURE);
  }

  // Write to file
  for (int i = 0; i < MAX_QUADON; i++) {
    for (int j = 0; j < MAX_CODON; j++) {
      gzwrite(binding_matrix_file, &instance_->binding_matrix_[i][j], sizeof(instance_->binding_matrix_[i][j]));
    }
  }

  gzclose(binding_matrix_file);
}

void BindingMatrix::load() {
  assert(instance_ == nullptr);

  instance_ = std::make_unique<BindingMatrix>();

  // Open file
  gzFile binding_matrix_file = gzopen(binding_matrix_file_name.c_str(), "r");
  if (binding_matrix_file == Z_NULL) {
    std::cout << "ERROR : Could not read binding matrix file " << binding_matrix_file_name << std::endl;
    exit(EXIT_FAILURE);
  }

  // Read from file
  for (int i = 0; i < MAX_QUADON; i++) {
    for (int j = 0; j < MAX_CODON; j++) {
      gzread(binding_matrix_file, &instance_->binding_matrix_[i][j], sizeof(instance_->binding_matrix_[i][j]));
    }
  }

  gzclose(binding_matrix_file);
}

}
