// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef AEVOL_BINDING_MATRIX_H_
#define AEVOL_BINDING_MATRIX_H_

#include <cstdlib>
#include <iostream>
#include <memory>

#include "JumpingMT.h"
#include "macros.h"
#include "fuzzytypes.h"

namespace aevol {

using binding_intensity_t = double;

/// Binding Matrix for regulation network
///
/// Meant to be use primarily as a pseudo-singleton through the static functions only
class BindingMatrix {
 private:
  static std::unique_ptr<BindingMatrix> instance_;
  BindingMatrix (const BindingMatrix&) = delete;
  BindingMatrix (BindingMatrix&&) = delete;
  BindingMatrix& operator= (const BindingMatrix&) = delete;
  BindingMatrix& operator= (BindingMatrix&&) = delete;
 public:
  static const BindingMatrix& instance() {return *instance_;};

  BindingMatrix() = default;
  virtual ~BindingMatrix() = default;

  static void random_binding_matrix(double binding_zeros_percentage, std::shared_ptr<JumpingMT> prng);
  static void save();
  static void load();
  static inline binding_intensity_t get_binding(int row, int col);

 protected:
  binding_intensity_t binding_matrix_[MAX_QUADON][MAX_CODON];
};

binding_intensity_t BindingMatrix::get_binding(int row, int col)
{
  assert(instance_);

  if (row >= MAX_QUADON || col >= MAX_CODON) {
    std::cout << "OUT OF BOUND BINDING R " << row << " C " << col << std::endl;
    exit(EXIT_FAILURE);
  }

  return instance_->binding_matrix_[row][col];
}

}  // namespace aevol

#endif  // AEVOL_BINDING_MATRIX_H_
