// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************


#include "Protein_7.h"
#ifdef __REGUL
  #include "legacy/raevol/Protein_R.h"
#endif
#include "Rna_7.h"

namespace aevol {

#ifndef __REGUL
Protein_7::Protein_7(int32_t t_protein_start,
                     int32_t t_protein_end,
                     int32_t t_protein_length,
                     Strand strand,
                     double t_e,
                     Rna_7* rna) {
#else
Protein_7::Protein_7(int32_t t_protein_start,
                     int32_t t_protein_end,
                     int32_t t_protein_length,
                     Strand strand,
                     double t_e,
                     Rna_7* rna,
                     bool translated) {
#endif
  protein_start   = t_protein_start;
  protein_end     = t_protein_end;
  protein_length  = t_protein_length;
  strand_ = strand;
  e               = t_e;
  is_init_        = true;

  rna_list_.push_back(rna);

  #ifdef __REGUL
  rna->protein_list_.push_back(this);
  initial_e_ = e;
  translated_ = translated;
  #endif
}

#ifdef __REGUL
Protein_7* Protein_7::make_inherited_protein(Protein_7* model) {
  // signal proteins cannot be inherited
  assert(not model->signal_);

  Protein_7* protein = new Protein_7();
  protein->protein_start   = model->protein_start;
  protein->protein_end     = model->protein_end;
  protein->protein_length  = model->protein_length;
  protein->strand_ = model->strand_;
  protein->w = model->w;
  protein->m = model->m;
  protein->h = model->h;
  protein->e = model->e;
  // TODO<dpa> is_functional not copied -> check
  protein->is_init_ = true;

  for (int i = 0; i < model->nb_codons_; i++) {
    protein->codon_list[i] = model->codon_list[i];
  }
  protein->nb_codons_      = model->nb_codons_;

  // TODO<dpa> is_TF_ not copied -> check
  // TODO<dpa> delta_concentration_ not copied -> check
  protein->inherited_ = true;
  protein->signal_ = model->signal_; // = false ?
  protein->translated_ = false;

  protein->initial_e_ = model->initial_e_;
  // TODO<dpa> protein_id_ not copied -> check
  // TODO<dpa> network_id not copied -> check

  return protein;
}

Protein_7* Protein_7::make_signal_protein(Protein_7* model) {
  Protein_7* protein = new Protein_7();
  protein->protein_start   = model->protein_start;
  protein->protein_end     = model->protein_end;
  protein->protein_length  = model->protein_length;
  protein->strand_ = model->strand_;
  protein->w = model->w;
  protein->m = model->m;
  protein->h = model->h;
  protein->e = model->e;
  // TODO<dpa> is_functional not copied -> check
  protein->is_init_ = model->is_init_;

  for (int i = 0; i < model->nb_codons_; i++) {
    protein->codon_list[i] = model->codon_list[i];
  }
  protein->nb_codons_      = model->nb_codons_;

  // TODO<dpa> is_TF_ not copied -> check
  // TODO<dpa> delta_concentration_ not copied -> check
  protein->inherited_ = false;
  protein->signal_ = true;
  protein->translated_ = false;

  protein->initial_e_ = model->initial_e_;
  // TODO<dpa> protein_id_ not copied -> check
  // TODO<dpa> network_id not copied -> check

  return protein;
}

Protein_7* Protein_7::make_signal_protein_from_legacy(Protein_R* model) {
  Protein_7* protein = new Protein_7();

  protein->protein_length = model->length();
  protein->e = model->concentration();

  protein->signal_ = model->is_signal();

  protein->nb_codons_ = model->codons().size();

  protein->w = 0;
  protein->m = 0;
  protein->h = 0;

  for (int i = 0 ; i < protein->nb_codons_ ; i++)
    protein->codon_list[i] = model->get_codon(i)->value();

  return protein;
}
#endif
    bool Protein_7::operator<(const Protein_7& other){
      // printf("P_THIS %f %f %f %f :: %d %d %d\n",h,w,m,e,is_init_,signal_,inherited_);
      // printf("P_OTHER %f %f %f %f :: %d %d %d\n",other.h,other.w,other.m,other.e,is_init_,signal_,inherited_);
        return (h <  other.h)
               || (h == other.h && m < other.m)
               || (h == other.h && m == other.m && w < other.w)
               || (h == other.h && m == other.m && w == other.w && e < other.e);
    }

Protein_7::~Protein_7() {
  rna_list_.clear();
}

    // bool Protein_7::operator<(const Protein_7& other){



    //     return (h <  other.h)
    //            || (h == other.h && m < other.m)
    //            || (h == other.h && m == other.m && w < other.w)
    //            || (h == other.h && m == other.m && w == other.w && e < other.e);
    // }

}