// ****************************************************************************
//
//          Aevol - An in silico experimental evolution platform
//
// ****************************************************************************
//
// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
// Web: http://www.aevol.fr/
// E-mail: See <http://www.aevol.fr/contact/>
// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************


#ifndef AEVOL_RNA_7_H
#define AEVOL_RNA_7_H

#include <cstdint>

#include <list>
#include <vector>

#include "ae_enums.h"
#include "macros.h"
#include "Promoter.h"
#include "Strand.h"
#include "Utils.h"

namespace aevol {

class Individual_7;
class Protein_7;

class AffinityFactor {
 public:
  AffinityFactor(Protein_7* prot, double efactor, double ofactor) {
    protein = prot;
    enhancer_factor = efactor;
    operator_factor = ofactor;
  }

  double concentration();

  Protein_7* protein;
  double enhancer_factor;
  double operator_factor;
};


class Rna_7 {
 public:
 /*
  RNA Constructor
  */
  Rna_7() = default;
  Rna_7(int32_t t_begin,
        int32_t t_end,
        Strand t_leading_lagging,
        double t_e,
        int32_t t_length,
        Promoter* promo = nullptr);
  Rna_7(const Rna_7& clone);
  Rna_7(Rna_7* clone, Promoter* promo = nullptr);
  ~Rna_7();

void init(Promoter* prom_x, int32_t t_begin, int32_t t_end, Strand strand, double t_e, int32_t t_length);

#ifdef __REGUL
  std::vector<AffinityFactor> affinity_list;

    int nb_influences_ = 0;

  int32_t enhancer_position(int32_t length);
  int32_t operator_position(int32_t length);
  double compute_synthesis_rate();

  double affinity_with_protein(int32_t index, Protein_7* protein, Individual_7* indiv);
#endif

  /*
    Promoter back compatibility
  */ 
  int32_t pos  = -1;
  int8_t error = -1;
  bool leading_or_lagging;

  /*
    RNA variables
  */
  int32_t begin = -1;
  int32_t end = -1;
  Strand strand_;
  double e = -1.0;
  std::list<int32_t> start_prot;
  std::list<Protein_7*> protein_list_;
  int32_t start_prot_count_ = 0;
  int32_t length = -1;
  bool is_coding_ = false;

  bool is_init_ = false;
  bool to_compute_end_ = true;
  bool to_compute_start_pos_ = true;

  Promoter* prom = nullptr;

  int32_t network_id = -2;
};

}

#endif //AEVOL_RNA_7_H
